#include <cstdlib>
#include <fstream>
#include <iostream>

#include "graph.h"

#include <misc/utils.h>


using namespace CSA;


int
main(int argc, char** argv)
{
  std::cout << "Determinize and merge automata" << std::endl;
  std::cout << std::endl;
  if(argc < 3)
  {
    std::cout << "Usage: determinize automaton1 [automaton2 [..]] base_name" << std::endl;
    return 1;
  }
  std::cout << "Number of automata: " << (argc - 2) << std::endl;
  std::string base_name = argv[argc - 1];
  std::cout << "Base name: " << base_name << std::endl;
  std::cout << std::endl;

  double start = readTimer();
  Graph* result = 0;
  for(int arg = 1; arg < argc - 1; arg++)
  {
    std::string automaton_name(argv[arg]);
    std::cout << "Automaton: " << automaton_name << std::endl;
    Graph* graph = new Graph(automaton_name, true);
    if(!(graph->ok)) { delete graph; std::cout << std::endl; continue; }
    graph->printInfo();
//    std::cout << "Max label: " << graph->maxLabel() << std::endl;
    if(graph->isDeterministic())
    {
      std::cout << "The automaton is already deterministic." << std::endl;
    }
    else
    {
      std::cout << "Determinizing..."; std::cout.flush();
      graph->determinize();
      std::cout << " done!" << std::endl;
      graph->printInfo();
    }
    if(result == 0) { result = graph; }
    else            { result->addGraph(*graph); delete graph; }
    std::cout << std::endl;
  }
  if(result != 0)
  {
    std::cout << "Final graph:" << std::endl;
    result->printInfo();
    std::cout << std::endl;
    result->write(base_name);
    delete result;
  }
  double stop = readTimer();

  std::cout << "Used " << (stop - start) << " seconds." << std::endl;
  std::cout << "Memory: " << memoryUsage() << " kB" << std::endl;
  std::cout << std::endl;

  return 0;
}
